/**
 * @file libkylog.h
 * @author chengxiaocui (chengxiaocui@kylinos.cn)
 * @brief 安全SDK日志(外部使用)
 * @version 0.1
 * @date 2024-03-19
 * 
 * @copyright Copyright (c) 2024
 * @defgroup 安全SDK日志
 * @{
 */
#ifndef _LIBKYLOGSEC_H
#define _LIBKYLOGSEC_H

#include <sys/types.h>

#define KLOG_ERROR      3
#define KLOG_WARNING    4
#define KLOG_NOTICE     5
#define KLOG_INFO       6
#define KLOG_DEBUG      7

#ifdef __cplusplus
extern "C" {
#endif

typedef enum _kdk_log_audit_type {
	KDK_LOG_AUDIT_TYPE_UNKNOWN = 0,
	KDK_LOG_AUDIT_TYPE_CALL,
    KDK_LOG_AUDIT_TYPE_EXIT
} kdk_log_audit_type;

typedef enum _kdk_log_interface_type {
	KDK_LOG_INTERFACE_TYPE_C = 0,
	KDK_LOG_INTERFACE_TYPE_DBUS
} kdk_log_interface_type;

typedef enum _kdk_sec_log_status {
    KDK_SEC_LOG_DISABLE = 0,
    KDK_SEC_LOG_ENABLE,
} kdk_sec_log_status;

struct kdk_log_message;

/**
 * @brief 记录C接口非审计日志
 * 
 */
#define klog_c(level, module, format, ...)do {\
	struct kdk_log_message *klog_node = kdk_log_init_params(level, module, -1, -1, KDK_LOG_AUDIT_TYPE_UNKNOWN, KDK_LOG_INTERFACE_TYPE_C, NULL, NULL, NULL, __FUNCTION__);\
    kdk_log_write(klog_node, format, ##__VA_ARGS__);\
	kdk_log_release(klog_node);\
}while(0)

/**
 * @brief 记录C接口调用审计日志
 * 
 */
#define klog_c_call(module)do {\
	struct kdk_log_message *klog_node = kdk_log_init_params(KLOG_INFO, module, -1, -1, KDK_LOG_AUDIT_TYPE_CALL, KDK_LOG_INTERFACE_TYPE_C, NULL, NULL, NULL, __FUNCTION__);\
    kdk_log_write(klog_node, NULL);\
	kdk_log_release(klog_node);\
}while(0)

/**
 * @brief 记录C接口退出审计日志
 * 
 */
#define klog_c_exit(module)do {\
	struct kdk_log_message *klog_node = kdk_log_init_params(KLOG_INFO, module, -1, -1, KDK_LOG_AUDIT_TYPE_EXIT, KDK_LOG_INTERFACE_TYPE_C, NULL, NULL, NULL, __FUNCTION__);\
    kdk_log_write(klog_node, NULL);\
	kdk_log_release(klog_node);\
}while(0)

/**
 * @brief 记录DBus接口非审计日志
 * 
 */
#define klog_dbus(level, module, pid, uid, dbus_name, dbus_path, dbus_interface, func, format, ...)do {\
	struct kdk_log_message *klog_node = kdk_log_init_params(level, module, pid, uid, KDK_LOG_AUDIT_TYPE_UNKNOWN, KDK_LOG_INTERFACE_TYPE_DBUS, dbus_name, dbus_path, dbus_interface, func);\
    kdk_log_write(klog_node, format, ##__VA_ARGS__);\
	kdk_log_release(klog_node);\
}while(0)

/**
 * @brief 记录Dbus接口调用审计日志
 * 
 */
#define klog_dbus_call(module, pid, uid, dbus_name, dbus_path, dbus_interface, func)do {\
	struct kdk_log_message *klog_node = kdk_log_init_params(KLOG_INFO, module, pid, uid, KDK_LOG_AUDIT_TYPE_CALL, KDK_LOG_INTERFACE_TYPE_DBUS, dbus_name, dbus_path, dbus_interface, func);\
    kdk_log_write(klog_node, NULL);\
	kdk_log_release(klog_node);\
}while(0)

/**
 * @brief 记录DBus接口退出审计日志
 * 
 */
#define klog_dbus_exit(module, pid, uid, dbus_name, dbus_path, dbus_interface, func)do {\
	struct kdk_log_message *klog_node = kdk_log_init_params(KLOG_INFO, module, pid, uid, KDK_LOG_AUDIT_TYPE_EXIT, KDK_LOG_INTERFACE_TYPE_DBUS, dbus_name, dbus_path, dbus_interface, func);\
    kdk_log_write(klog_node, NULL);\
	kdk_log_release(klog_node);\
}while(0)

/**
 * @brief 设置安全日志开关
 * 
 * @param status 日志开关状态，参见 kdk_sec_log_status
 * @return int 0 设置成功; -1 设置失败
 */
extern int kdk_sec_log_set_status(int status);

/**
 * @brief 获取安全日志开关
 * 
 * @return int 0 关闭；1 开启；-1 获取失败
 */
extern int kdk_sec_log_get_status();

/**
 * @brief 设置日志等级
 * 
 * @param level 日志等级，范围[3,7]，只记录<=level等级的日志
 * @return int 0 设置成功；-1 设置失败
 */
extern int kdk_sec_log_set_level(int level);

/**
 * @brief 获取日志等级
 * 
 * @return int [3,7] 对应日志等级；-1 获取失败
 */
extern int kdk_sec_log_get_level();


/**
 * @brief 创建并初始化审计结构体（内部使用函数，不应在外部被调用）
 * 
 */
struct kdk_log_message *kdk_log_init_params(int level, const char* module_name, pid_t pid, uid_t uid, int audit_type, int interface_type, const char *dbus_name, const char *dbus_path, const char *dbus_interface, const char *func);

/**
 * @brief 释放审计信息结构体（内部使用函数，不应在外部被调用）
 * 
 */
void kdk_log_release(struct kdk_log_message *klog_mesg);

/**
 * @brief 发送审计信息（内部使用函数，不应在外部被调用）
 * 
 */
void kdk_log_write(struct kdk_log_message *klog_mesg, const char* format, ...);

#ifdef __cplusplus
}
#endif
#endif
